-- GET_GUI.E Graphical "get"
-- Daniel Berstein, 19 of April 1997
-- See instruccions in GET_GUI.DOC

include putsxy.e
include get.e

--Declare variables
global sequence boxes
object aux

boxes = ""

procedure beep()
    sound(800)
    aux = time()
    while time() <= aux+0.3 do
    end while
    sound(0)
end procedure

procedure frame(integer col1,integer col2,sequence coord1,sequence coord2)
    draw_line(col1,coord1)
    draw_line(col2,coord2)
end procedure

global procedure kill_fields()
    boxes = ""
end procedure

global procedure draw_field(integer xpos,integer ypos,integer len,integer c1,
  integer c2,integer f_fr,integer f_bg,integer uf_fr)

    --Add a new field
    boxes = append(boxes, {xpos,ypos,len,c1,c2,f_fr,f_bg,uf_fr})

    --Draw the new field
    polygon(f_bg,1,{{xpos,ypos},{xpos+(len*8)+8,ypos},{xpos+(len*8)+8,ypos+24},
	{xpos,ypos+24}})
    frame(c1,c2,{{xpos,ypos+24},{xpos,ypos},{xpos+(len*8)+8,ypos}},
	{{xpos,ypos+24},{xpos+(len*8)+8,ypos+24},{xpos+(len*8)+8,ypos}})
    frame(c1,c2,{{xpos+1,ypos+23},{xpos+1,ypos+1},{xpos+(len*8)+7,ypos+1}},
	{{xpos+1,ypos+23},{xpos+(len*8)+7,ypos+23},{xpos+(len*8)+7,ypos+1}})
end procedure

function read_one_field(sequence box_info, sequence text)
    integer xpos,ypos,len,c1,c2,fr,bg
    sequence input
    integer char

    --Get field information
    xpos = box_info[1]
    ypos = box_info[2]
    len = box_info[3]
    c1 = box_info[4]
    c2 = box_info[5]
    fr = box_info[6]
    bg = box_info[7]
    
    --Get field contents
    input = text

    --Main loop
    char = wait_key()
    while char != 13 do
	--Back space?
	if char = 8 then
	    if length(input) > 1 then
		input = input[1..length(input)-1]
	    else
		input = ""
	    end if
	    putsxy({xpos+((length(input))*8)+4,ypos+4}," ",fr,bg,'a')
	--ESC?
	elsif char = 27 then
	    if length(input) = 0 then
		--If field clear, return
		return {input,1}
	    else
		--Clear field... next ESC will return
		input = ""
		putsxy({xpos+4,ypos+4},repeat(32,len),fr,bg,'a')
	    end if
	--TAB?
	elsif char = 9 then
	    --Return
	    return {input,2}
	--All OK, append character
	elsif length(input) < len then
	    input = input & char
	--Too many characters... beep!
	else
	    beep()
	end if

	--Display character
	if length(input) <= len and length(input) > 0 then
	    putsxy({xpos+((length(input)-1)*8)+4,ypos+4},{input[length(input)]},
	      fr,bg,'a')
	end if
	--Wait for a keystroke
	char = wait_key()
    end while
    --If ENTER pressed, return
    return {input,0}
end function

global function read_field(sequence text)
    integer index, tab, xpos, ypos, len, c1, c2
    object input

    --If there're no fields defined, return NUL string
    if length(boxes) < 1 then
	return ""
    end if

    --If TEXT mismatches number of boxes, add NUL strings or slice TEXT
    if length(text) < length(boxes) then
	for loop = 1 to length(boxes) - length(text) do
	    text = text & ""
	end for
    elsif length(text) > length(boxes) then
	text = text[1..length(boxes)]
    end if

    --Display field contents
    for loop = 1 to length(boxes) do
	putsxy({boxes[loop][1]+4,boxes[loop][2]+4},text[loop],
	  boxes[loop][8],boxes[loop][7],'a')
    end for
    
    --Start with the first field
    index = 1
    --Main loop
    while 1 do
	--Get field information
	xpos = boxes[index][1]
	ypos = boxes[index][2]
	len = boxes[index][3]
	c1 = boxes[index][4]
	c2 = boxes[index][5]

	--Make "active" field
	frame(6,6,{{xpos,ypos+24},{xpos,ypos},{xpos+(len*8)+8,ypos}},
	    {{xpos,ypos+24},{xpos+(len*8)+8,ypos+24},{xpos+(len*8)+8,ypos}})
	putsxy({boxes[index][1]+4,boxes[index][2]+4},text[index],
	  boxes[index][6],boxes[index][7],'a')
	
	--Get text from the field!
	input = read_one_field(boxes[index],text[index])
	text[index] = input[1]
	if input[2] then
	    if input[2] = 1 then
		return text
	    else
		tab = 1
	    end if
	else
	    tab = 0
	end if
	
	--Mark as unfocus the field
	frame(c1,c2,{{xpos,ypos+24},{xpos,ypos},{xpos+(len*8)+8,ypos}},
	    {{xpos,ypos+24},{xpos+(len*8)+8,ypos+24},{xpos+(len*8)+8,ypos}})
	putsxy({boxes[index][1]+4,boxes[index][2]+4},text[index],
	  boxes[index][8],boxes[index][7],'a')
	
	--Go to next field
	index = index + 1
	--Is it the last field?
	if index > length(boxes) then
	    --If TAB, then recicle fields
	    if tab then
		index = 1
	    else
		return text
	    end if          
	end if
    end while
end function    
